<?php

/*
Plugin Name: ZedPress Shortcode Converter
Description: Converts the old ZStore shortcode to the new ZPress shortcode
Author: Paul Robinson
Version: 1.0
Author URI: https://return-true.com
*/

// No classes here, just a standard PHP script
function zedpress_convert($items = 0, $per_page = 30) {

    // Make sure the WP $shortcode_tags is available
    global $shortcode_tags;

    // Create array of old attributes and their new eqivs
    $opts = array(
        'contributorhandle' => 'store_name',
        'associateid' => 'referral',
        'productlineid' => '',
        'producttype' => 'department',
        'keywords' => 'keywords',
        'gridwidth' => '',
        'gridcellsize' => '',
        'gridcellspacing' => '',
        'gridcellbgcolor' => '',
        'useanalytics' => '',
        'analyticsid' => '',
        'showhowmany' => 'per_page',
        'startpage' => 'page',
        'showpagination' => '',
        'showsorting' => '',
        'defaultsort' => 'sort_type',
        'showproductdescription' => '',
        'showbyline' => '',
        'showproducttitle' => '',
        'showproductprice' => '',
        'usecaching' => '',
        'cachelifetime' => '',
    );

    // Store all of the shortcodes registered with WordPress
    $tmp_shrt = $shortcode_tags;

    // Remove all shortcodes and only add the one we want to find
    $shortcode_tags = array( 'zstore' => '' );

    // Generate the regex for that shortcode
    $regex = '/' . get_shortcode_regex() . '/s';

    // Restore the stored shortcodes
    $shortcode_tags = $tmp_shrt;

    // Create a query of posts
    $query = new WP_Query(array(
        'posts_per_page' => $per_page,
        'offset' => $items, // Offset used to paginate
        'post_type' => 'post',
        'post_status' => 'publish'
    ));

    // Loop through the posts
    if( $query->have_posts() ) :
        while( $query->have_posts() ) :
            $query->the_post();

            // Get the raw content with shortcodes intact
            $content = get_the_content();

            // Create our shortcode replacement array here
            // Will be emptied each new post
            $shorts = array();

            // Match for our old ZStore shortcode
            if( preg_match_all( $regex, $content, $matches) && array_key_exists( 2, $matches ) && in_array( 'zstore', $matches[2] ) ) {

                foreach( $matches[2] as $key => $match ) {

                    // If we had a match let's check for attributes of the matching key
                    $atts = shortcode_parse_atts( $matches[3][$key] );
                    $newAtts = '';

                    if( !empty( $atts ) ) {
                        // If there were attributes run through the attributes array and match them
                        // Create a new attribute string from the matching items
                        // Remove empty items as they are no longer applicable

                        foreach( $atts as $att_key => $att_val ) {
                            if( array_key_exists( $att_key, $opts ) ) {
                                // Debug line
                                // print_r( "Convert {$att_key} to {$opts[$att_key]} value was {$att_val}" );
                                $newAtts .= $opts[$att_key] . '="' . esc_attr( $att_val ) . '"';
                            }
                        }
                    }

                    // Push the new shortcode onto our array with the old shortcode as the key
                    $shorts[$matches[0][$key]] = "[zedpress {$newAtts}]";

                }

            }

            // Run the replacement on the post content
            $content = str_replace( array_keys( $shorts ), array_values( $shorts ), $content );

            $update = array(
                'ID' => get_the_id(),
                'post_content' => $content
            );

            $success = wp_update_post( $update, true );

        endwhile;
    else :
        // If there are no posts left return false
        return false;
    endif;

    return true;

}

add_action( 'admin_menu', 'zedpress_convert_admin_menu' );

function zedpress_convert_admin_menu() {
	add_menu_page( 'Zed Press Converter', 'Zed Press Convert', 'manage_options', 'zedpress/zedpress-convert.php', 'zedpress_convert_admin_page' );
}

function zedpress_convert_admin_page(){
    $do = filter_input( INPUT_GET, 'zedpressconvert' );
    if( $do ) {
        // Items per page
        $per_page = 30;

        // Item we have just processed
        $items = filter_input( INPUT_GET, 'zpcovitems' );
        $items = $items + $per_page;

        $status = zedpress_convert($items, $per_page);

    }
	?>
	<div class="wrap">
		<h2>Zed Press Shortcode Converter</h2>
        <?php echo ( isset( $notice ) ? $notice : '' ); ?>
        <p>
            <strong>WARNING:</strong> This conversion plugin is designed to irreversibly change post content. As with any plugin that makes these kind of destuctive changes please make sure you have a backup of your WordPress content before running it. While the plugin has been tested and only matches a specific type shortcode it is possible, although unlikely, that something could go wrong so please backup first.
        </p>

        <p>
            The converter will change your post content by replacing older <code>[zstore]</code> shortcodes with new <code>[zedpress]</code> shortcodes. Attributes on the shortcode will be converted to their new equivlents except where they are no longer applicable or have been removed. Those attributes will be removed.
        </p>

        <p>
            Made a backup? Yes? Great. Click the button below to run the converter. It will run through all posts and pages.
        </p>

        <?php if(filter_input( INPUT_GET, 'zedpressfinished', FILTER_VALIDATE_BOOLEAN )) : ?>
            <p>
                <strong>Conversion is complete!</strong>
            </p>
            <p>
                You can run the conversion again below, although there should not be any need too.
            </p>
        <?php endif; ?>

        <?php if(!$do) : ?>
            <p>
                <a href="<?php echo admin_url('admin.php?page=zedpress/zedpress-convert.php&zedpressconvert=1&zpcovitems=0'); ?>" class="button-primary">Run Conversion</a>
            </p>
        <?php else : ?>
            <h3>Status</h3>
            <p>
                Completed <?php echo $items; ?>
            </p>
            <?php if($status) : // If status is true there are posts left ?>
                <p>
                    Moving on to next batch in 3 seconds.
                    <script type="text/javascript">
                        setTimeout(function() {
                            window.location = '<?php echo admin_url('admin.php?page=zedpress/zedpress-convert.php&zedpressconvert=1&zpcovitems=' . $items); ?>';
                        }, 3000);
                    </script>
                </p>
            <?php else: // Status is false no posts left, redirect to finish message ?>
                <script type="text/javascript">
                    setTimeout(function() {
                        window.location = '<?php echo admin_url('admin.php?page=zedpress/zedpress-convert.php&zedpressfinished=1'); ?>';
                    }, 3000);
                </script>
            <?php endif; ?>
        <?php endif; ?>
	</div>
	<?php
}
